@@ -68,7 +68,11 @@ const Uint32 CORE_UPDATE_INTERVAL = 250;
 KTorrentCore::KTorrentCore(kt::GUIInterface* gui) : max_downloads(0),keep_seeding(true),pman(0)
 {
 	UpdateCurrentTime();
+	
 	qman = new QueueManager();
+	connect(qman, SIGNAL(lowDiskSpace(kt::TorrentInterface*,bool)), this, SLOT(onLowDiskSpace(kt::TorrentInterface*,bool)));
+	
+	
 	data_dir = Settings::tempDir();
 	bool dd_not_exist = !bt::Exists(data_dir);
 	if (data_dir == QString::null || dd_not_exist)
@@ -143,38 +147,79 @@ bool KTorrentCore::init(TorrentControl* tc,bool silently)
 {
 	connectSignals(tc);
 	qman->append(tc);
-	if (tc->getStats().multi_file_torrent && !silently)
+	
+	bool user = false;
+	bool start_torrent = false;
+	
+	if (!silently)
 	{
-		FileSelectDlg dlg;
+		FileSelectDlg dlg(gman, &user, &start_torrent);
 
 		if (dlg.execute(tc) != QDialog::Accepted)
 		{
-			if (!tc->hasExistingFiles())
-				remove(tc,true);
-			else
-				remove(tc,false);
+			remove(tc,false);
 			return false;
 		}
-	}
+	}	
 	
-	tc->createFiles();
+	try
+	{
+		tc->createFiles();
+	}
+	catch (bt::Error & err)
+	{
+		// if we can't create files, just remove the torrent
+		KMessageBox::error(0,err.toString());
+		remove(tc,false);
+		return false;
+	}
 		
 	if (tc->hasExistingFiles())
 	{
 		ScanDialog* scan_dlg = new ScanDialog(this,true);
 		scan_dlg->show();
 		scan_dlg->execute(tc,true);
-	//	scan_dlg->deleteLater();
 	}
-		
+	
 	tc->setPreallocateDiskSpace(true);
 
 	if(Settings::maxRatio()>0)
-        	tc->setMaxShareRatio(Settings::maxRatio()); 
+		tc->setMaxShareRatio(Settings::maxRatio()); 
+	
+	if (Settings::maxSeedTime() > 0)
+		tc->setMaxSeedTime(Settings::maxSeedTime());
 	
 	torrentAdded(tc);
-	qman->torrentAdded(tc);
+	qman->torrentAdded(tc, user, start_torrent);
 		
+	
+	//now copy torrent file to user specified dir if needed
+	if(Settings::useTorrentCopyDir())
+	{
+		QString torFile = tc->getTorDir();
+		
+		if(!torFile.endsWith("/"))
+			torFile += "/";
+		
+		torFile += "torrent";
+		
+		QString destination = Settings::torrentCopyDir();
+		
+		if(!destination.endsWith("/"))
+			destination += "/";
+		
+		destination += tc->getStats().torrent_name + ".torrent";
+		
+		try
+		{
+			bt::CopyFile(torFile, destination, TRUE);
+		}
+		catch(bt::Error& err)
+		{
+			Out(SYS_GEN|LOG_IMPORTANT) << "Could not copy torrent file to " << destination << endl;
+		}
+	}
+	
 	return true;
 }
 
@@ -256,9 +301,8 @@ void KTorrentCore::downloadFinished(KIO::Job *job)
 	{
 		// load in the file (target is always local)
 		QString dir = Settings::saveDir();
-		if (!Settings::useSaveDir())
-			dir = KFileDialog::getExistingDirectory(QString::null, 0,
-				i18n("Select Folder to Save To"));
+		if (!Settings::useSaveDir() || dir.isNull())
+			dir = QDir::homeDirPath();
 	
 		if (dir != QString::null && load(j->data(),dir,false, j->url()))
 			loadingFinished(j->url(),true,false);
@@ -273,9 +317,8 @@ void KTorrentCore::load(const KURL& url)
 	{
 		QString path = url.path(); 
 		QString dir = Settings::saveDir();
-		if (!Settings::useSaveDir())
-			dir = KFileDialog::getExistingDirectory(QString::null, 0,
-				i18n("Select folder for data of %1").arg(url.prettyURL()));
+		if (!Settings::useSaveDir() || dir.isNull())
+			dir = QDir::homeDirPath();
 	
 		if (dir != QString::null && load(path,dir,false))
 			loadingFinished(url,true,false);
@@ -296,28 +339,37 @@ void KTorrentCore::downloadFinishedSilently(KIO::Job *job)
 	if (err == KIO::ERR_USER_CANCELED)
 	{
 		loadingFinished(j->url(),false,true);
-		return;
 	}
-	
-	if (err)
+	else if (err)
 	{
 		loadingFinished(j->url(),false,false);
 	}
 	else
 	{
-		// load in the file (target is always local)
-		QString dir = Settings::saveDir();
-		if (!Settings::useSaveDir())
+		QString dir;
+		if (custom_save_locations.contains(j))
 		{
-			loadingFinished(j->url(),false,false);
+			// we have a custom save location so save to that
+			dir = custom_save_locations[j].path();
+			custom_save_locations.erase(j);
+		}
+		else if (!Settings::useSaveDir())
+		{
+			// incase save dir is not set, use home director
+			Out(SYS_GEN|LOG_NOTICE) << "Cannot load " << j->url() << " silently, default save location not set !" << endl;
+			Out(SYS_GEN|LOG_NOTICE) << "Using home directory instead !" << endl;
+			dir = QDir::homeDirPath();
 		}
 		else
 		{
-			if (dir != QString::null && load(j->data(),dir,true,j->url()))
-				loadingFinished(j->url(),true,false);
-			else
-				loadingFinished(j->url(),false,false);
+			dir = Settings::saveDir();
 		}
+		
+		
+		if (dir != QString::null && load(j->data(),dir,true,j->url()))
+			loadingFinished(j->url(),true,false);
+		else
+			loadingFinished(j->url(),false,false);
 	}
 }
 
@@ -327,7 +379,7 @@ void KTorrentCore::loadSilently(const KURL& url)
 	{
 		QString path = url.path(); 
 		QString dir = Settings::saveDir();
-		if (!Settings::useSaveDir())
+		if (!Settings::useSaveDir() || dir.isNull())
 		{
 			Out(SYS_GEN|LOG_NOTICE) << "Cannot load " << path << " silently, default save location not set !" << endl;
 			Out(SYS_GEN|LOG_NOTICE) << "Using home directory instead !" << endl;
@@ -347,9 +399,54 @@ void KTorrentCore::loadSilently(const KURL& url)
 	}
 }
 
+void KTorrentCore::loadSilentlyDir(const KURL& url, const KURL& savedir)
+{
+	if (url.isLocalFile())
+	{
+		QString path = url.path(); 
+		QString dir = savedir.path();
+		QFileInfo fi(dir);
+		if (!fi.exists() || !fi.isWritable() || !fi.isDir())
+		{
+			Out(SYS_GEN|LOG_NOTICE) << "Cannot load " << path << " silently, destination directory is not OK ! Using default save directory." << endl;
+			dir = Settings::saveDir();
+			if (!Settings::useSaveDir())
+			{
+				Out(SYS_GEN|LOG_NOTICE) << "Default save directory not set, using home directory !" << endl;
+				dir = QDir::homeDirPath();
+			}
+		}
+	 	
+		if (dir != QString::null && load(path,dir,true))
+			loadingFinished(url,true,false);
+		else
+			loadingFinished(url,false,true);
+	}
+	else
+	{
+		// download to a random file in tmp
+		KIO::Job* j = KIO::storedGet(url,false,true);
+		custom_save_locations.insert(j,savedir); // keep track of save location
+		connect(j,SIGNAL(result(KIO::Job*)),this,SLOT(downloadFinishedSilently( KIO::Job* )));
+	}
+}
+
 void KTorrentCore::start(kt::TorrentInterface* tc)
 {
-	qman->start(tc);
+	kt::TorrentStartResponse reason = qman->start(tc);
+	switch (reason)
+	{
+		// we can return, the question to ignore the limits will have informed the user
+		case MAX_SHARE_RATIO_REACHED: 
+		case START_OK:  // start OK is normal 
+		case BUSY_WITH_DATA_CHECK: // checking data, so let the torrent be
+		case USER_CANCELED:
+			return;
+		case NOT_ENOUGH_DISKSPACE:
+		case QM_LIMITS_REACHED:
+			canNotStart(tc,reason);
+			break;
+	}
 }
 
 void KTorrentCore::stop(TorrentInterface* tc, bool user)
@@ -495,14 +592,12 @@ void KTorrentCore::onExit()
 	qman->onExit(job);
 	// wait for completion of stopped events
 	if (job->needToWait())
-	{
 		WaitJob::execute(job);
-	}
 	else
 		delete job;
 	
-	qman->clear();
 	pman->unloadAll(false);
+	qman->clear();
 }
 
 bool KTorrentCore::changeDataDir(const QString & new_dir)
@@ -593,7 +688,6 @@ void KTorrentCore::stopAll(int type)
 void KTorrentCore::update()
 {
 	bt::UpdateCurrentTime();
-	KT_PROF_START("core");
 	AuthenticationMonitor::instance().update();
 	
 	QPtrList<kt::TorrentInterface>::iterator i = qman->begin();
@@ -626,7 +720,6 @@ void KTorrentCore::update()
 		}
 		i++;
 	}
-	KT_PROF_END();
 }
 
 void KTorrentCore::makeTorrent(const QString & file,const QStringList & trackers,
@@ -702,7 +795,7 @@ CurrentStats KTorrentCore::getStats()
 
 bool KTorrentCore::changePort(Uint16 port)
 {
-	if (qman->count() == 0)
+	if (getNumTorrentsRunning() == 0)
 	{
 		Globals::instance().getServer().changePort(port);
 		return true;
@@ -755,10 +848,13 @@ bt::QueueManager* KTorrentCore::getQueueManager()
 	return this->qman;
 }
 
-void KTorrentCore::torrentSeedAutoStopped( kt::TorrentInterface * tc )
+void KTorrentCore::torrentSeedAutoStopped( kt::TorrentInterface * tc,kt::AutoStopReason reason )
 {
 	qman->startNext();
-	maxShareRatioReached(tc);
+	if (reason == kt::MAX_RATIO_REACHED)
+		maxShareRatioReached(tc);
+	else
+		maxSeedTimeReached(tc);
 }
 
 int KTorrentCore::getMaxUploadSpeed()
@@ -970,7 +1066,7 @@ void KTorrentCore::aboutToBeStarted(kt::TorrentInterface* tc,bool & ret)
 	else
 	{
 		QString msg = i18n("The file where the data is saved of the torrent \"%1\" is missing, do you want to recreate it?").arg(tc->getStats().torrent_name);
-		int ret = KMessageBox::warningYesNo(0,msg, i18n("Recreate"), KStdGuiItem::cancel());
+		int ret = KMessageBox::warningYesNo(0,msg, i18n("Recreate"),KGuiItem(i18n("Recreate")),KGuiItem(i18n("Do Not Recreate")));
 		if (ret == KMessageBox::Yes)
 		{
 			try
@@ -1005,14 +1101,17 @@ void KTorrentCore::connectSignals(kt::TorrentInterface* tc)
 			this,SLOT(torrentFinished(kt::TorrentInterface* )));
 	connect(tc, SIGNAL(stoppedByError(kt::TorrentInterface*, QString )),
 			this, SLOT(slotStoppedByError(kt::TorrentInterface*, QString )));
-	connect(tc, SIGNAL(seedingAutoStopped( kt::TorrentInterface* )),
-			this, SLOT(torrentSeedAutoStopped( kt::TorrentInterface* )));
+	connect(tc, SIGNAL(seedingAutoStopped(kt::TorrentInterface*, kt::AutoStopReason)),
+			this, SLOT(torrentSeedAutoStopped(kt::TorrentInterface*, kt::AutoStopReason)));
 	connect(tc,SIGNAL(aboutToBeStarted( kt::TorrentInterface*,bool & )),
 			this, SLOT(aboutToBeStarted( kt::TorrentInterface*,bool & )));
 	connect(tc,SIGNAL(corruptedDataFound( kt::TorrentInterface* )),
 			this, SLOT(emitCorruptedData( kt::TorrentInterface* )));
-	connect(qman, SIGNAL(queuingNotPossible( kt::TorrentInterface* )),
+	connect(qman, SIGNAL(queuingNotPossible(kt::TorrentInterface*)),
 			this, SLOT(enqueueTorrentOverMaxRatio( kt::TorrentInterface* )));
+	connect(qman, SIGNAL(lowDiskSpace(kt::TorrentInterface*, bool)),
+			this, SLOT(onLowDiskSpace(kt::TorrentInterface*, bool)));
+	
 }
 
 float KTorrentCore::getGlobalMaxShareRatio() const
@@ -1038,4 +1137,9 @@ void KTorrentCore::doDataCheck(kt::TorrentInterface* tc)
 	scan_dlg->execute(tc,false);
 }
 
+void KTorrentCore::onLowDiskSpace(kt::TorrentInterface * tc, bool stopped)
+{
+	emit lowDiskSpace(tc, stopped);
+}
+
 #include "ktorrentcore.moc"
