@@ -23,6 +23,7 @@
 
 #include <util/ptrmap.h>
 #include "cache.h"
+#include "settings.h"
 
 namespace bt
 {
@@ -53,9 +54,13 @@ namespace bt
 		virtual void close();
 		virtual void open();
 		virtual QString getOutputPath() const;
+		virtual void changeOutputPath(const QString & outputpath);
+		virtual KIO::Job* moveDataFiles(const QString & ndir);
+		virtual void moveDataFilesCompleted(KIO::Job* job);
 		virtual void preallocateDiskSpace(PreallocationThread* prealloc);
 		virtual bool hasMissingFiles(QStringList & sl);
 		virtual void deleteDataFiles();
+		virtual Uint64 diskUsage();
 	private:
 		void touch(TorrentFile & tf);
 		virtual void downloadStatusChanged(TorrentFile*, bool);
