@@ -26,18 +26,24 @@
 
 namespace bt
 {
+	char RandomLetterOrNumber()
+	{
+		int i = rand() % 62;
+		if (i < 26)
+			return 'a' + i;
+		else if (i < 52)
+			return 'A' + (i - 26);
+		else
+			return '0' + (i - 52);
+	}
+
 	
 	PeerID::PeerID()
 	{
 		srand(time(0));
-		int r[12];
-		for (int i = 0;i < 12;i++)
-			r[i] = rand() % 10;
-		QString peer_id = kt::PEER_ID;
-		for (int i = 0;i < 12;i++)
-			peer_id += QString("%1").arg(r[i]);
-		memcpy(id,peer_id.ascii(),20);
-
+		memcpy(id,kt::PEER_ID,8);
+		for (int i = 8;i < 20;i++)
+			id[i] = RandomLetterOrNumber(); 
 		client_name = identifyClient();
 	}
 
@@ -114,37 +120,73 @@ namespace bt
 		{
 			// Keep things a bit alphabetic to make it easier add new ones
 			//AZUREUS STYLE
+			Map["AG"] = "Ares";
+			Map["A~"] = "Ares";
+			Map["AV"] = "Avicora";
+			Map["AX"] = "BitPump";
 			Map["AR"] = "Arctic";
 			Map["AZ"] = "Azureus";
-			Map["BC"] = "BitComet";
 			Map["BB"] = "BitBuddy";
+			Map["BC"] = "BitComet";
+			Map["BF"] = "Bitflu";
 			Map["BG"] = "BTGetit";
 			Map["BM"] = "BitMagnet";
+			Map["BO"] = "BitsOnWheels";
+			Map["BR"] = "BitRocket";
 			Map["BS"] = "BTSlave"; 
 			Map["BX"] = "BitTorrent X";
+			Map["CD"] = "Enhanced CTorrent";
 			Map["CT"] = "CTorrent";
+			Map["DE"] = "DelugeTorrent";
+			Map["DP"] = "Propagate Data Client";
+			Map["EB"] = "EBit";
+			Map["ES"] = "electric sheep";
+			Map["FT"] = "FoxTorrent";
+			Map["GS"] = "GSTorrent";
 			Map["G3"] = "G3 Torrent";
+			Map["HL"] = "Halite";
+			Map["HN"] = "Hydranode";
+			Map["KG"] = "KGet";
 			Map["KT"] = "KTorrent"; // lets not forget our own client
+			Map["LH"] = "LH-ABC";
 			Map["lt"] = "libTorrent";
 			Map["LT"] = "libtorrent";
 			Map["LP"] = "Lphant";
+			Map["LW"] = "LimeWire";
 			Map["ML"] = "MLDonkey";
+			Map["MO"] = "MonoTorrent";
 			Map["MP"] = "MooPolice";
 			Map["MT"] = "MoonLight";
+			Map["PD"] = "Pando";
+			Map["qB"] = "qBittorrent";
+			Map["QD"] = "QQDownload";
 			Map["QT"] = "Qt 4 Torrent example";
 			Map["RS"] = "Rufus";
+			Map["RT"] = "Retriever";
+			Map["S~"] = "Shareaza alpha/beta";
 			Map["SB"] = "Swiftbit";
 			Map["SS"] = "SwarmScope";
+			Map["ST"] = "SymTorrent";
+			Map["st"] = "sharktorrent";
 			Map["SZ"] = "Shareaza";
 			Map["TN"] = "Torrent .NET";
 			Map["TR"] = "Transmission";
 			Map["TS"] = "Torrent Storm";
+			Map["TT"] = "TuoTu";
+			Map["UL"] = "uLeecher!";
 			Map["UT"] = QString("%1Torrent").arg(QChar(0x00B5)); // µTorrent, 0x00B5 is unicode for µ
+			Map["WT"] = "BitLet";
+			Map["WY"] = "FireTorrent";
+			Map["XL"] = "Xunlei";
 			Map["XT"] = "Xan Torrent";
+			Map["XX"] = "Xtorrent";
 			Map["ZT"] = "Zip Torrent";
 			
 			//SHADOWS STYLE
 			Map["A"] = "ABC";
+			Map["O"] = "Osprey Permaseed";
+			Map["Q"] = "BTQueue";
+			Map["R"] = "Tribler";
 			Map["S"] = "Shadow's";
 			Map["T"] = "BitTornado";
 			Map["U"] = "UPnP NAT BitTorrent";
