@@ -20,6 +20,10 @@
 #ifndef BTFILEOPS_H
 #define BTFILEOPS_H
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <util/constants.h>
 class QString;
 
@@ -111,12 +115,13 @@ namespace bt
 	 * @param size The size to truncate to
 	 * @throw Error if the file doesn't exist, or something else goes wrong
 	 */
-	void TruncateFile(int fd,Uint64 size);
+	void TruncateFile(int fd,Uint64 size,bool quick);
 	
 	/**
 	 * Truncate a file (wrapper around ftruncate)
 	 * @param fd Path of the file
 	 * @param size The size to truncate to
+	 * @param quick Use the quick way (doesn't prevent fragmentationt)
 	 * @throw Error if the file doesn't exist, or something else goes wrong
 	 */
 	void TruncateFile(const QString & path,Uint64 size);
@@ -130,7 +135,20 @@ namespace bt
 	 * Special truncate for FAT file systems.
 	 */
 	bool FatPreallocate(const QString & path,Uint64 size);
+
+#ifdef HAVE_XFS_XFS_H
+	/**
+	 * Special truncate for XFS file systems.
+	*/
+	bool XfsPreallocate(int fd,Uint64 size);
 	
+	/**
+	 * Special truncate for XFS file systems.
+	 */
+	bool XfsPreallocate(const QString & path,Uint64 size);
+
+#endif
+
 	/**
 	 * Seek in a file, wrapper around lseek
 	 * @param fd The file descriptor
@@ -139,6 +157,9 @@ namespace bt
 	 * @throw Error if something else goes wrong
 	 */
 	void SeekFile(int fd,Int64 off,int whence);
+
+	/// Calculate the number of bytes free on the filesystem path is located
+	bool FreeDiskSpace(const QString & path,Uint64 & bytes_free);
 }
 
 #endif
