@@ -36,14 +36,15 @@
 #include <ksqueezedtextlabel.h>
 #include <kglobal.h>
 #include <kiconloader.h>
+#include <klistview.h>
 
 #include <torrent/globals.h>
 #include <util/log.h>
 
 namespace kt
 {
-	TrackerView::TrackerView(TorrentInterface* ti, QWidget *parent, const char *name)
-		:TrackerViewBase(parent, name), tc(ti)
+	TrackerView::TrackerView(QWidget *parent, const char *name)
+		:TrackerViewBase(parent, name), tc(0)
 	{
 		KIconLoader* iload = KGlobal::iconLoader();
 		btnUpdate->setIconSet(iload->loadIconSet("apply", KIcon::Small));
@@ -51,30 +52,13 @@ namespace kt
 		btnRemove->setIconSet(iload->loadIconSet("remove", KIcon::Small));
 		btnRestore->setIconSet(iload->loadIconSet("undo", KIcon::Small));
 		
-		if(!tc)
-			return;
-		
-		const KURL::List trackers = tc->getTrackersList()->getTrackerURLs();
-		if(trackers.empty())
-			return;
-		
-		for (KURL::List::const_iterator i = trackers.begin();i != trackers.end();i++)
-			new QListViewItem(listTrackers, (*i).prettyURL());
-		
-		if (tc->getStats().priv_torrent)
-		{
-			btnAdd->setEnabled(false);
-			btnRemove->setEnabled(false);
-			btnRestore->setEnabled(false);
-			btnChange->setEnabled(true);
-			txtTracker->setText(i18n("You cannot add trackers to a private torrent"));
-			txtTracker->setEnabled(false);
-		}
-		else if (!tc->getStats().running)
-		{
-			btnUpdate->setEnabled(false);
-			btnChange->setEnabled(false);
-		}
+		QPalette p = lblCurrent->palette();
+		p.setColor(QPalette::Active,QColorGroup::Base,p.color(QPalette::Active,QColorGroup::Background));
+		lblCurrent->setPalette(p);
+	}
+	
+	TrackerView::~TrackerView()
+	{
 	}
 
 	void TrackerView::btnAdd_clicked()
@@ -155,9 +139,18 @@ namespace kt
 			txtTracker->setText(item->text(0));
 	}
 	
-	void TrackerView::update(TorrentInterface* ti)
+	void TrackerView::changeTC(TorrentInterface* ti)
+	{
+		if (tc == ti)
+			return;
+		
+		setEnabled(ti != 0);
+		torrentChanged(ti);
+		update();
+	}
+	
+	void TrackerView::update()
 	{
-		tc = ti;
 		if(!tc)
 			return;
 		
@@ -176,13 +169,22 @@ namespace kt
 
 		lblStatus->setText("<b>" + s.trackerstatus + "</b>");
 		if (tc->getTrackersList())
-			lblCurrent->setText("<b>" + tc->getTrackersList()->getTrackerURL().prettyURL() + "</b>");
+		{
+			QString t = tc->getTrackersList()->getTrackerURL().prettyURL();
+			if (lblCurrent->text() != t )
+				lblCurrent->setText(t);
+		}
 		else
 			lblCurrent->clear();
 		
 		btnAdd->setEnabled(txtTracker->text() != QString::null && !tc->getStats().priv_torrent);
 	}
 	
+	void TrackerView::onLoadingFinished(const KURL & ,bool,bool)
+	{
+		torrentChanged(tc);
+	}
+	
 	void TrackerView::torrentChanged(TorrentInterface* ti)
 	{
 		tc = ti;
