@@ -19,19 +19,19 @@
 		echo "<FORM method=\"GET\">";
 		echo "<tr>";
 			echo "<td>Upload Speed (0 is no limit): </td>";
-			echo "<td><INPUT type=\"text\" name=\"maximum_upload_rate\" value=\"$globalinfo[max_upload_speed]\"></td>";
+			echo "<td><INPUT type=\"text\" name=\"maximum_upload_rate\" value=\"{$globalinfo['max_upload_speed']}\"></td>";
 		echo " </tr>";
 		echo "<tr>";
 			echo "<td>Download Speed (0 is no limit): </td>";
-			echo "<td><INPUT type=\"text\" name=\"maximum_download_rate\" value=\"$globalinfo[max_download_speed]\"></td>";
+			echo "<td><INPUT type=\"text\" name=\"maximum_download_rate\" value=\"{$globalinfo['max_download_speed']}\"></td>";
 		echo "</tr>";
 		echo "<tr>";
 			echo "<td>Maximum downloads (0 is no limit): </td>";
-			echo "<td><INPUT type=\"text\" name=\"maximum_downloads\" value=\"$globalinfo[max_downloads]\"></td>";
+			echo "<td><INPUT type=\"text\" name=\"maximum_downloads\" value=\"{$globalinfo['max_downloads']}\"></td>";
 		echo"</tr>";
 		echo "<tr>";
 			echo "<td>Maximum seeds (0 is no limit): </td>";
-			echo "<td><INPUT type=\"text\" name=\"maximum_seeds\" value=\"$globalinfo[max_seeds]\"></td>";
+			echo "<td><INPUT type=\"text\" name=\"maximum_seeds\" value=\"{$globalinfo['max_seeds']}\"></td>";
 		echo"</tr>";
 		echo "<tr><td><INPUT type=\"submit\"></tr></td>";
 		echo "</FORM>";
